@@ -2,7 +2,7 @@
  * mpeg_ts.c : Transport Stream input module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: ts.c,v 1.14 2003/01/08 16:40:29 fenrir Exp $
+ * $Id: ts.c,v 1.17 2003/02/12 13:42:43 massiot Exp $
  *
  * Authors: Henri Fallon <henri@via.ecp.fr>
  *          Johan Bilien <jobi@via.ecp.fr>
@@ -100,6 +100,11 @@ static void TS_DVBPSI_HandlePMT ( input_thread_t *, dvbpsi_pmt_t * );
     "0.3.x and 0.4. By default VLC assumes you have the latest VLS. In case " \
     "you're using an old version, select this option.")
 
+#define BUGGY_PSI_TEXT N_("buggy PSI")
+#define BUGGY_PSI_LONGTEXT N_( \
+    "If you have a stream whose PSI packets do not feature incremented " \
+    "continuity counters, select this option.")
+
 vlc_module_begin();
 #if defined MODULE_NAME_IS_ts
     set_description( _("ISO 13818-1 MPEG Transport Stream input") );
@@ -113,6 +118,7 @@ vlc_module_begin();
     add_category_hint( N_("Miscellaneous"), NULL );
     add_bool( "vls-backwards-compat", 0, NULL,
               VLS_BACKWARDS_COMPAT_TEXT, VLS_BACKWARDS_COMPAT_LONGTEXT );
+    add_bool( "buggy-psi", 0, NULL, BUGGY_PSI_TEXT, BUGGY_PSI_LONGTEXT );
     set_callbacks( Activate, Deactivate );
 vlc_module_end();
 
@@ -192,6 +198,7 @@ static int Activate( vlc_object_t * p_this )
 
     p_stream_data = (stream_ts_data_t *)p_input->stream.p_demux_data;
     p_stream_data->i_pat_version = PAT_UNINITIALIZED ;
+    p_stream_data->b_buggy_psi = config_GetInt( p_input, "buggy-psi" );
 
 #ifdef MODULE_NAME_IS_ts_dvbpsi
     p_stream_data->p_pat_handle = (dvbpsi_handle *)
@@ -216,6 +223,7 @@ static int Activate( vlc_object_t * p_this )
     p_demux_data->i_psi_type = PSI_IS_PAT;
     p_demux_data->p_psi_section = malloc(sizeof(psi_section_t));
     p_demux_data->p_psi_section->b_is_complete = 1;
+    p_demux_data->i_continuity_counter = 0xFF;
 
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
@@ -532,6 +540,7 @@ static void TSDecodePAT( input_thread_t * p_input, es_descriptor_t * p_es )
                     p_es_demux->p_psi_section =
                                             malloc( sizeof( psi_section_t ) );
                     p_es_demux->p_psi_section->b_is_complete = 0;
+                    p_es_demux->i_continuity_counter = 0xFF;
                 }
             }
 
@@ -623,6 +632,7 @@ static void TSDecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                 /* Add this ES to the program */
                 p_new_es = input_AddES( p_input, p_es->p_pgrm,
                                         (u16)i_pid, sizeof( es_ts_data_t ) );
+                ((es_ts_data_t *)p_new_es->p_demux_data)->i_continuity_counter = 0xFF;
 
                 /* Tell the interface what kind of stream it is and select
                  * the required ones */
@@ -631,11 +641,17 @@ static void TSDecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                     {
                         case MPEG1_VIDEO_ES:
                         case MPEG2_VIDEO_ES:
+                            /* This isn't real, but we don't actually use
+                             * it. */
+                            p_new_es->i_stream_id = 0xE0;
                             p_new_es->i_fourcc = VLC_FOURCC('m','p','g','v');
                             p_new_es->i_cat = VIDEO_ES;
                             break;
                         case MPEG1_AUDIO_ES:
                         case MPEG2_AUDIO_ES:
+                            /* This isn't real, but we don't actually use
+                             * it. */
+                            p_new_es->i_stream_id = 0xC0;
                             p_new_es->i_fourcc = VLC_FOURCC('m','p','g','a');
                             p_new_es->i_cat = AUDIO_ES;
                             break;
@@ -724,6 +740,13 @@ static void TSDecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                     p_input->pf_set_program( p_input, p_es->p_pgrm );
         }
 
+        /* if the pmt belongs to the currently selected program, we
+         * reselect it to update its ES */
+        else if( p_es->p_pgrm == p_input->stream.p_selected_program )
+        {
+            p_input->pf_set_program( p_input, p_es->p_pgrm );
+        }
+
         /* inform interface that stream has changed */
         p_input->stream.b_changed = 1;
         /*  Remove lock */
@@ -750,20 +773,20 @@ static void TS_DVBPSI_DemuxPSI( input_thread_t  * p_input,
     pgrm_ts_data_t      * p_pgrm_demux_data;
     stream_ts_data_t    * p_stream_demux_data;
 
-    p_es_demux_data = ( es_ts_data_t * ) p_es->p_demux_data;
-    p_stream_demux_data = ( stream_ts_data_t * ) p_input->stream.p_demux_data;
+    p_es_demux_data = (es_ts_data_t *)p_es->p_demux_data;
+    p_stream_demux_data = (stream_ts_data_t *) p_input->stream.p_demux_data;
 
     switch( p_es_demux_data->i_psi_type)
     {
         case PSI_IS_PAT:
             dvbpsi_PushPacket(
-                    ( dvbpsi_handle ) p_stream_demux_data->p_pat_handle,
+                    (dvbpsi_handle)p_stream_demux_data->p_pat_handle,
                     p_data->p_demux_start );
             break;
         case PSI_IS_PMT:
             p_pgrm_demux_data = ( pgrm_ts_data_t * )p_es->p_pgrm->p_demux_data;
             dvbpsi_PushPacket(
-                    ( dvbpsi_handle ) p_pgrm_demux_data->p_pmt_handle,
+                    (dvbpsi_handle)p_pgrm_demux_data->p_pmt_handle,
                     p_data->p_demux_start );
             break;
         default:
@@ -1142,7 +1165,7 @@ static void TS_DVBPSI_HandlePAT( input_thread_t * p_input,
 
     p_stream_data = (stream_ts_data_t *)p_input->stream.p_demux_data;
 
-    if ( !p_new_pat->b_current_next ||
+    if ( ( p_new_pat->b_current_next && ( p_new_pat->i_version != p_stream_data->i_pat_version ) ) ||
             p_stream_data->i_pat_version == PAT_UNINITIALIZED  )
     {
         /* Delete all programs */
@@ -1191,6 +1214,7 @@ static void TS_DVBPSI_HandlePAT( input_thread_t * p_input,
                 }
 
                 p_es_demux->p_psi_section->b_is_complete = 0;
+                p_es_demux->i_continuity_counter = 0xFF;
 
                 /* Create a PMT decoder */
                 p_pgrm_demux->p_pmt_handle = (dvbpsi_handle *)
@@ -1240,7 +1264,7 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
     p_pgrm_demux = (pgrm_ts_data_t *)p_pgrm->p_demux_data;
     p_pgrm_demux->i_pcr_pid = p_new_pmt->i_pcr_pid;
 
-    if( !p_new_pmt->b_current_next ||
+    if( ( p_new_pmt->b_current_next && ( p_new_pmt->i_version != p_pgrm_demux->i_pmt_version ) ) ||
             p_pgrm_demux->i_pmt_version == PMT_UNINITIALIZED )
     {
         dvbpsi_descriptor_t *p_dr = p_new_pmt->p_first_descriptor;
@@ -1265,6 +1289,8 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
                 p_input->b_error = 1;
                 return;
             }
+            ((es_ts_data_t *)p_new_es->p_demux_data)->i_continuity_counter = 0xFF;
+
             switch( p_es->i_type )
             {
                 case MPEG1_VIDEO_ES:
