@@ -2,7 +2,7 @@
  * controls.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: controls.m,v 1.19 2003/01/31 02:53:52 jlj Exp $
+ * $Id: controls.m,v 1.28 2003/02/12 14:22:23 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -53,12 +53,18 @@
 - (IBAction)next:(id)sender;
 - (IBAction)loop:(id)sender;
 
+- (IBAction)forward:(id)sender;
+- (IBAction)backward:(id)sender;
+
 - (IBAction)volumeUp:(id)sender;
 - (IBAction)volumeDown:(id)sender;
 - (IBAction)mute:(id)sender;
 - (IBAction)volumeSliderUpdated:(id)sender;
 - (void)updateVolumeSlider;
 
+- (IBAction)halfWindow:(id)sender;
+- (IBAction)normalWindow:(id)sender;
+- (IBAction)doubleWindow:(id)sender;
 - (IBAction)fullscreen:(id)sender;
 - (IBAction)deinterlace:(id)sender;
 
@@ -272,23 +278,47 @@
     vlc_object_release( p_playlist );
 }
 
+- (IBAction)forward:(id)sender
+{
+    intf_thread_t * p_intf = [NSApp getIntf];
+    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                                       FIND_ANYWHERE );
+    if( p_playlist == NULL || p_playlist->p_input == NULL )
+    {
+        if ( p_playlist != NULL ) vlc_object_release( p_playlist );
+        return;
+    }
+
+    input_Seek( p_playlist->p_input, 5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+    vlc_object_release( p_playlist );
+}
+
+- (IBAction)backward:(id)sender
+{
+    intf_thread_t * p_intf = [NSApp getIntf];
+    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                                       FIND_ANYWHERE );
+    if( p_playlist == NULL || p_playlist->p_input == NULL )
+    {
+        if ( p_playlist != NULL ) vlc_object_release( p_playlist );
+        return;
+    }
+
+    input_Seek( p_playlist->p_input, -5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+    vlc_object_release( p_playlist );
+}
+
 - (IBAction)volumeUp:(id)sender
 {
     intf_thread_t * p_intf = [NSApp getIntf];
 
-    aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                        FIND_ANYWHERE );
-    if( p_aout != NULL )
+    if( p_intf->p_sys->b_mute )
     {
-        if( p_intf->p_sys->b_mute )
-        {
-            [self mute: nil];
-        }
-
-        aout_VolumeUp( p_aout, 1, NULL );
-        vlc_object_release( (vlc_object_t *)p_aout );
+        [self mute: nil];
     }
 
+    aout_VolumeUp( p_intf, 1, NULL );
+
     [self updateVolumeSlider];
 }
 
@@ -296,18 +326,12 @@
 {
     intf_thread_t * p_intf = [NSApp getIntf];
 
-    aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                        FIND_ANYWHERE );
-    if( p_aout != NULL )
+    if( p_intf->p_sys->b_mute )
     {
-        if( p_intf->p_sys->b_mute )
-        {
-            [self mute: nil];
-        }
-
-        aout_VolumeDown( p_aout, 1, NULL );
-        vlc_object_release( (vlc_object_t *)p_aout );
+        [self mute: nil];
     }
+    
+    aout_VolumeDown( p_intf, 1, NULL );
 
     [self updateVolumeSlider];
 }
@@ -315,19 +339,10 @@
 - (IBAction)mute:(id)sender
 {
     intf_thread_t * p_intf = [NSApp getIntf];
+    audio_volume_t i_volume;
 
-    aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                        FIND_ANYWHERE );
-
-    if ( p_aout != NULL )
-    {
-        audio_volume_t i_volume;
-
-        aout_VolumeMute( p_aout, &i_volume );
-        vlc_object_release( (vlc_object_t *)p_aout );
-
-        p_intf->p_sys->b_mute = ( i_volume == 0 );
-    }
+    aout_VolumeMute( p_intf, &i_volume );
+    p_intf->p_sys->b_mute = ( i_volume == 0 );
 
     [self updateVolumeSlider];
 }
@@ -335,38 +350,67 @@
 - (IBAction)volumeSliderUpdated:(id)sender
 {
     intf_thread_t * p_intf = [NSApp getIntf];
+    audio_volume_t i_volume = (audio_volume_t)[sender intValue];
 
-    aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                        FIND_ANYWHERE );
-    if( p_aout != NULL )
-    {
-        audio_volume_t i_volume;
-
-        i_volume = (audio_volume_t)[sender intValue];
-
-        aout_VolumeSet( p_aout, i_volume * AOUT_VOLUME_STEP );
-        vlc_object_release( (vlc_object_t *)p_aout );
-    }
+    aout_VolumeSet( p_intf, i_volume * AOUT_VOLUME_STEP );
 }
 
 - (void)updateVolumeSlider
 {
     intf_thread_t * p_intf = [NSApp getIntf];
+    audio_volume_t i_volume;
 
-    aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                        FIND_ANYWHERE );
+    aout_VolumeGet( p_intf, &i_volume );
 
-    if ( p_aout != NULL )
+    [o_volumeslider setFloatValue: (float)(i_volume / AOUT_VOLUME_STEP)]; 
+}
+
+- (IBAction)halfWindow:(id)sender
+{
+    id o_window = [NSApp keyWindow];
+    NSArray *o_windows = [NSApp windows];
+    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
+    
+    while ((o_window = [o_enumerator nextObject]))
     {
-        audio_volume_t i_volume;
-
-        aout_VolumeGet( p_aout, &i_volume );
-        vlc_object_release( (vlc_object_t *)p_aout );
-
-        [o_volumeslider setFloatValue: (float)(i_volume / AOUT_VOLUME_STEP)]; 
+        if( [[o_window className] isEqualToString: @"VLCWindow"] )
+        {
+            [o_window scaleWindowWithFactor: 0.5];
+        }
     }
 }
 
+- (IBAction)normalWindow:(id)sender
+{
+    id o_window = [NSApp keyWindow];
+    NSArray *o_windows = [NSApp windows];
+    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
+    
+    while ((o_window = [o_enumerator nextObject]))
+    {
+        if( [[o_window className] isEqualToString: @"VLCWindow"] )
+        {
+            [o_window scaleWindowWithFactor: 1];
+        }
+    }
+}
+
+- (IBAction)doubleWindow:(id)sender
+{
+    id o_window = [NSApp keyWindow];
+    NSArray *o_windows = [NSApp windows];
+    NSEnumerator *o_enumerator = [o_windows objectEnumerator];
+    
+    while ((o_window = [o_enumerator nextObject]))
+    {
+        if( [[o_window className] isEqualToString: @"VLCWindow"] )
+        {
+            [o_window scaleWindowWithFactor: 2];
+        }
+    }
+}
+
+
 - (IBAction)fullscreen:(id)sender
 {
     id o_window = [NSApp keyWindow];
@@ -384,7 +428,19 @@
 
 - (IBAction)deinterlace:(id)sender
 {
-
+    intf_thread_t * p_intf = [NSApp getIntf];
+    BOOL bEnable = [sender state] == NSOffState;
+    
+    if( bEnable )
+    {
+        config_PutPsz( p_intf, "filter", "deinterlace" );
+	config_PutPsz( p_intf, "deinterlace-mode",
+                    [[sender title] lossyCString] );
+    }
+    else
+    {
+        config_PutPsz( p_intf, "filter", NULL );
+    }
 }
 
 - (IBAction)toggleProgram:(id)sender
@@ -520,7 +576,7 @@
         NSOpenPanel *o_open_panel = [NSOpenPanel openPanel];
         
         [o_open_panel setAllowsMultipleSelection: NO];
-        [o_open_panel setTitle: _NS("Open subtitlefile")];
+        [o_open_panel setTitle: _NS("Open subtitle file")];
         [o_open_panel setPrompt: _NS("Open")];
     
         if( [o_open_panel runModalForDirectory: nil 
@@ -586,7 +642,7 @@
     if( [o_mi state] == NSOffState )
     {
         const char * psz_variable = (const char *)[o_mi tag];
-        const char * psz_value = [[o_mi title] cString];
+        char * psz_value = [NSApp delocalizeString: [o_mi title]];
         vlc_object_t * p_object = (vlc_object_t *)
             [[o_mi representedObject] pointerValue];
         vlc_value_t val;
@@ -597,6 +653,8 @@
         {
             msg_Warn( p_object, "cannot set variable (%s)", psz_value );
         }
+
+        free( psz_value );
     }
 }
 
@@ -667,16 +725,14 @@
 
         [o_mi setState: i_state];
     }
-    else if( [[o_mi title] isEqualToString: _NS("Volume Up")] ||
-             [[o_mi title] isEqualToString: _NS("Volume Down")] ) 
+    else if( [[o_mi title] isEqualToString: _NS("Step Forward")] ||
+             [[o_mi title] isEqualToString: _NS("Step Backward")] )
     {
-        aout_instance_t * p_aout;
-
-        p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                          FIND_ANYWHERE );
-        if( p_aout != NULL )
+        if( p_playlist != NULL && p_input != NULL )
         {
-            vlc_object_release( (vlc_object_t *)p_aout );
+            vlc_mutex_lock( &p_input->stream.stream_lock );
+            bEnabled = p_input->stream.b_seekable;
+            vlc_mutex_unlock( &p_input->stream.stream_lock );
         }
         else
         {
@@ -685,22 +741,12 @@
     }
     else if( [[o_mi title] isEqualToString: _NS("Mute")] ) 
     {
-        aout_instance_t * p_aout;
-
-        p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                          FIND_ANYWHERE );
-        if( p_aout != NULL )
-        {
-            vlc_object_release( (vlc_object_t *)p_aout );
-        }
-        else
-        {
-            bEnabled = FALSE;
-        }
-
         [o_mi setState: p_intf->p_sys->b_mute ? NSOnState : NSOffState];
     }
-    else if( [[o_mi title] isEqualToString: _NS("Fullscreen")] )    
+    else if( [[o_mi title] isEqualToString: _NS("Fullscreen")] ||
+                [[o_mi title] isEqualToString: _NS("Half Size")] ||
+                [[o_mi title] isEqualToString: _NS("Normal Size")] ||
+                [[o_mi title] isEqualToString: _NS("Double Size")])    
     {
         id o_window;
         NSArray *o_windows = [NSApp windows];
@@ -718,9 +764,34 @@
     }
     else if( o_menu != nil && 
              [[o_menu title] isEqualToString: _NS("Deinterlace")] )
-    { 
+    {
+        char * psz_filter = config_GetPsz( p_intf, "filter" );
+        
+        if( psz_filter != NULL )
+        {
+            free( psz_filter );
+            
+            psz_filter = config_GetPsz( p_intf, "deinterlace-mode" );
+        }
 
-    } 
+        if( psz_filter != NULL )
+        {
+            if( strcmp( psz_filter, [[o_mi title] lossyCString] ) == 0 )
+            {
+                [o_mi setState: NSOnState];
+            }
+            else
+            {
+                [o_mi setState: NSOffState];
+            }
+
+            free( psz_filter );
+        }
+        else
+        {
+            [o_mi setState: NSOffState];
+        }
+    }
 
     if( p_playlist != NULL )
     {
