@@ -2,7 +2,7 @@
  * intf.h: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: intf.h,v 1.20 2003/01/31 02:53:52 jlj Exp $
+ * $Id: intf.h,v 1.28 2003/02/16 23:32:06 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -41,6 +41,7 @@
 
 - (void)initIntlSupport;
 - (NSString *)localizedString:(char *)psz;
+- (char *)delocalizeString:(NSString *)psz;
 
 - (void)setIntf:(intf_thread_t *)p_intf;
 - (intf_thread_t *)getIntf;
@@ -65,8 +66,10 @@ struct intf_sys_t
 
     /* interface update */
     vlc_bool_t b_intf_update;
-    vlc_bool_t b_play_status; 
-
+    vlc_bool_t b_play_status;
+    vlc_bool_t b_playlist_update;
+    vlc_bool_t b_current_title_update;
+    
     /* menus handlers */
     vlc_bool_t b_chapter_update;
     vlc_bool_t b_program_update;
@@ -111,12 +114,14 @@ struct intf_sys_t
 
     IBOutlet id o_controls;     /* VLCControls    */
     IBOutlet id o_playlist;     /* VLCPlaylist    */
+    IBOutlet id o_info;		/* VLCInfo        */
 
     IBOutlet id o_messages;     /* messages tv    */
     IBOutlet id o_msgs_panel;   /* messages panel */
     IBOutlet id o_msgs_btn_ok;  /* messages btn   */
     NSMutableArray * o_msg_arr; /* messages array */
     NSLock * o_msg_lock;        /* messages lock  */
+    IBOutlet id o_msgs_btn_crashlog;	/* messages open crashlog */
 
     IBOutlet id o_error;        /* error panel    */
     IBOutlet id o_err_msg;      /* NSTextView     */
@@ -124,6 +129,8 @@ struct intf_sys_t
     IBOutlet id o_err_bug_lbl;
     IBOutlet id o_err_btn_msgs; /* Open Messages  */
     IBOutlet id o_err_btn_dismiss;
+    
+    IBOutlet id o_info_window;	/* Info panel     */
 
     /* main menu */
 
@@ -157,6 +164,8 @@ struct intf_sys_t
     IBOutlet id o_mi_previous;
     IBOutlet id o_mi_next;
     IBOutlet id o_mi_loop;
+    IBOutlet id o_mi_fwd;
+    IBOutlet id o_mi_bwd;
     IBOutlet id o_mi_program;
     IBOutlet id o_mi_title;
     IBOutlet id o_mi_chapter;
@@ -171,6 +180,9 @@ struct intf_sys_t
     IBOutlet id o_mi_device;
 
     IBOutlet id o_mu_video;
+    IBOutlet id o_mi_half_window;
+    IBOutlet id o_mi_normal_window;
+    IBOutlet id o_mi_double_window;
     IBOutlet id o_mi_fullscreen;
     IBOutlet id o_mi_screen;
     IBOutlet id o_mi_deinterlace;
@@ -180,6 +192,7 @@ struct intf_sys_t
     IBOutlet id o_mi_close_window;
     IBOutlet id o_mi_controller;
     IBOutlet id o_mi_playlist;
+    IBOutlet id o_mi_info;
     IBOutlet id o_mi_messages;
     IBOutlet id o_mi_bring_atf;
     
@@ -192,6 +205,8 @@ struct intf_sys_t
     /* dock menu */
     IBOutlet id o_dmi_play;
     IBOutlet id o_dmi_stop;
+    IBOutlet id o_dmi_next;
+    IBOutlet id o_dmi_previous;
 }
 
 - (void)terminate;
@@ -230,6 +245,7 @@ struct intf_sys_t
 - (IBAction)reportABug:(id)sender;
 - (IBAction)openWebsite:(id)sender;
 - (IBAction)openLicense:(id)sender;
+- (IBAction)openCrashLog:(id)sender;
 
 - (void)windowDidBecomeKey:(NSNotification *)o_notification;
 
