@@ -26,6 +26,7 @@
 #include <vlc/vlc.h>
 #include <vlc/intf.h>
 #include <vlc/vout.h>
+#include <vlc/aout.h>
 
 #include "dragdrop.h"
 #include "mainframe.h"
@@ -43,7 +44,12 @@
 
 #include "netutils.h"
 
-//---------------------------------------------------------------------------
+/*****************************************************************************
+ * BORLAND SUCKS
+ * If the following pragma doesn't end with \r\n instead of \n, the file
+ * doesn't compile anymore... Funny, isn't it?
+ * With vim, you can use ":set ff=dos" to correct it
+ *****************************************************************************/
 #pragma link "CSPIN"
 #pragma resource "*.dfm"
 
@@ -356,40 +362,18 @@ void __fastcall TMainFrameDlg::EjectActionExecute( TObject *Sender )
 //--------------------------------------------------------------------------
 void __fastcall TMainFrameDlg::VolumeUpActionExecute( TObject *Sender )
 {
-    aout_instance_t *p_aout;
-    p_aout = (aout_instance_t *)vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                 FIND_ANYWHERE );
-    if ( p_aout != NULL )
-    {
-        aout_VolumeUp( p_aout, 1, NULL );
-        vlc_object_release( (vlc_object_t *)p_aout );
-    }
+    aout_VolumeUp( p_intf, 1, NULL );
 }
 //---------------------------------------------------------------------------
 void __fastcall TMainFrameDlg::VolumeDownActionExecute( TObject *Sender )
 {
-    aout_instance_t *p_aout;
-    p_aout = (aout_instance_t *)vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                 FIND_ANYWHERE );
-    if ( p_aout != NULL )
-    {
-        aout_VolumeDown( p_aout, 1, NULL );
-        vlc_object_release( (vlc_object_t *)p_aout );
-    }
+    aout_VolumeDown( p_intf, 1, NULL );
 }
 //---------------------------------------------------------------------------
 void __fastcall TMainFrameDlg::MuteActionExecute( TObject *Sender )
 {
-    aout_instance_t *p_aout;
-    p_aout = (aout_instance_t *)vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                 FIND_ANYWHERE );
-    if ( p_aout != NULL )
-    {
-        aout_VolumeMute( p_aout, NULL );
-        vlc_object_release( (vlc_object_t *)p_aout );
-
-//        MenuMute->Checked = ! MenuMute->Checked;
-    }
+    aout_VolumeMute( p_intf, NULL );
+//    MenuMute->Checked = ! MenuMute->Checked;
 }
 //---------------------------------------------------------------------------
 
