@@ -1,8 +1,8 @@
 /*****************************************************************************
- * win32_specific.c: Win32 specific features 
+ * win32_specific.c: Win32 specific features
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: win32_specific.c,v 1.7.2.1 2002/07/29 16:12:24 gbazin Exp $
+ * $Id: win32_specific.c,v 1.21 2003/02/17 05:50:31 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -11,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,69 +21,94 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
-#include <errno.h>                                                 /* ENOMEM */
 #include <string.h>                                              /* strdup() */
 #include <stdlib.h>                                                /* free() */
-#include <fcntl.h>
 
-#include <winsock2.h>
+#include <vlc/vlc.h>
 
-#include <videolan/vlc.h>
+#if !defined( UNDER_CE )
+#   include <fcntl.h>
+#   include <winsock2.h>
+#endif
 
 /*****************************************************************************
  * system_Init: initialize winsock and misc other things.
  *****************************************************************************/
-void system_Init( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
+void system_Init( vlc_t *p_this, int *pi_argc, char *ppsz_argv[] )
 {
+#if !defined( UNDER_CE )
     WSADATA Data;
     int i_err;
-    HINSTANCE hInstLib;
 
-    /* Allocate structure */
-    p_main->p_sys = malloc( sizeof( main_sys_t ) );
-    if( p_main->p_sys == NULL )
+    /* Get our full path */
+    if( ppsz_argv[0] )
     {
-        intf_ErrMsg( "init error: can't create p_main->p_sys (%s)",
-		     strerror(ENOMEM) );
-        exit(-1);
-    }
+        char psz_path[MAX_PATH];
+        char *psz_vlc;
 
-    /* dynamically get the address of SignalObjectAndWait */
-    if( (GetVersion() < 0x80000000) )
+        GetFullPathName( ppsz_argv[0], MAX_PATH, psz_path, &psz_vlc );
+
+        if( psz_vlc > psz_path && psz_vlc[-1] == '\\' )
+        {
+            psz_vlc[-1] = '\0';
+            p_this->p_libvlc->psz_vlcpath = strdup( psz_path );
+        }
+        else
+        {
+            p_this->p_libvlc->psz_vlcpath = strdup( "" );
+        }
+    }
+    else
     {
-        /* We are running on NT/2K/XP, we can use SignalObjectAndWait */
-        hInstLib = LoadLibrary( "kernel32" );
-        if( hInstLib)
-            p_main->p_sys->SignalObjectAndWait =
-                (SIGNALOBJECTANDWAIT)GetProcAddress( hInstLib,
-                                                     "SignalObjectAndWait" );
+        p_this->p_libvlc->psz_vlcpath = strdup( "" );
     }
-    else p_main->p_sys->SignalObjectAndWait = NULL;
 
     /* WinSock Library Init. */
     i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
 
     if( i_err )
     {
-        fprintf( stderr, "error: can't initiate WinSocks, error %i", i_err );
+        fprintf( stderr, "error: can't initiate WinSocks, error %i\n", i_err );
     }
 
-    _fmode = _O_BINARY;  /* sets the default file-translation mode */
+    /* Set the default file-translation mode */
+    _fmode = _O_BINARY;
+#endif
 }
 
 /*****************************************************************************
  * system_Configure: check for system specific configuration options.
  *****************************************************************************/
-void system_Configure( void )
+void system_Configure( vlc_t *p_this )
 {
-    p_main->p_sys->b_fast_mutex = config_GetIntVariable( "fast-mutex" );
-    p_main->p_sys->i_win9x_cv = config_GetIntVariable( "win9x-cv-method" );
+#if !defined( UNDER_CE )
+    p_this->p_libvlc->b_fast_mutex = config_GetInt( p_this, "fast-mutex" );
+    p_this->p_libvlc->i_win9x_cv = config_GetInt( p_this, "win9x-cv-method" );
+
+    /* Raise default priority of the current process */
+#ifndef ABOVE_NORMAL_PRIORITY_CLASS
+#   define ABOVE_NORMAL_PRIORITY_CLASS 0x00008000
+#endif
+    if( !SetPriorityClass( GetCurrentProcess(),
+			   ABOVE_NORMAL_PRIORITY_CLASS ) )
+    {
+        if( !SetPriorityClass( GetCurrentProcess(),
+			       HIGH_PRIORITY_CLASS ) )
+	    msg_Dbg( p_this, "can't raise process priority" );
+	else
+	    msg_Dbg( p_this, "raised process priority" );
+    }
+    else
+	msg_Dbg( p_this, "raised process priority" );
+#endif
 }
 
 /*****************************************************************************
  * system_End: terminate winsock.
  *****************************************************************************/
-void system_End( void )
+void system_End( vlc_t *p_this )
 {
+#if !defined( UNDER_CE )
     WSACleanup();
+#endif
 }
