@@ -2,7 +2,7 @@
  * vout_pictures.c : picture management functions
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: vout_pictures.c,v 1.26 2002/05/28 18:34:42 stef Exp $
+ * $Id: vout_pictures.c,v 1.34 2003/01/22 10:44:50 fenrir Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -25,20 +25,21 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                                /* free() */
 #include <stdio.h>                                              /* sprintf() */
 #include <string.h>                                            /* strerror() */
 
-#include <videolan/vlc.h>
+#include <vlc/vlc.h>
 
 #include "video.h"
 #include "video_output.h"
 
+#include "vout_pictures.h"
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static void vout_CopyPicture( picture_t *p_src, picture_t *p_dest );
+static void CopyPicture( vout_thread_t *, picture_t *, picture_t * );
 
 /*****************************************************************************
  * vout_DisplayPicture: display a picture
@@ -59,8 +60,8 @@ void vout_DisplayPicture( vout_thread_t *p_vout, picture_t *p_pic )
         p_pic->i_status = READY_PICTURE;
         break;
     default:
-        intf_ErrMsg( "error: picture %p has invalid status %d",
-                     p_pic, p_pic->i_status );
+        msg_Err( p_vout, "picture to display %p has invalid status %d",
+                         p_pic, p_pic->i_status );
         break;
     }
 
@@ -88,8 +89,8 @@ void vout_DatePicture( vout_thread_t *p_vout,
         p_pic->i_status = READY_PICTURE;
         break;
     default:
-        intf_ErrMsg( "error: picture %p has invalid status %d",
-                     p_pic, p_pic->i_status );
+        msg_Err( p_vout, "picture to date %p has invalid status %d",
+                         p_pic, p_pic->i_status );
         break;
     }
 
@@ -105,9 +106,9 @@ void vout_DatePicture( vout_thread_t *p_vout,
  * since several pictures can be created by several producers threads.
  *****************************************************************************/
 picture_t *vout_CreatePicture( vout_thread_t *p_vout,
-                               boolean_t b_progressive,
-                               boolean_t b_top_field_first,
-                               boolean_t b_repeat_first_field )
+                               vlc_bool_t b_progressive,
+                               vlc_bool_t b_top_field_first,
+                               vlc_bool_t b_repeat_first_field )
 {
     int         i_pic;                                      /* picture index */
     picture_t * p_pic;
@@ -117,22 +118,12 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
     vlc_mutex_lock( &p_vout->picture_lock );
 
     /*
-     * Look for an empty place. We start at 1 because the first
-     * directbuffer is reserved for memcpy()ed pictures.
+     * Look for an empty place in the picture heap.
      */
-    for( i_pic = 0; i_pic < I_RENDERPICTURES && p_freepic == NULL; i_pic++ )
+    for( i_pic = 0; i_pic < I_RENDERPICTURES; i_pic++ )
     {
-        p_pic = PP_RENDERPICTURE[ i_pic ];
-
-        /* If the picture we found is a memory buffer, and we have enough
-         * pictures in the stack, and we might have enough room later for
-         * a direct buffer, skip it. If no other pictures are found, the
-         * video decoder will try again later. */
-        if( p_vout->b_direct && ( p_vout->output.i_pictures > 5 )
-             && ( p_pic->i_type != DIRECT_PICTURE ) )
-        {
-            break;
-        }
+        p_pic = PP_RENDERPICTURE[(p_vout->render.i_last_used_pic + i_pic + 1)
+                                 % I_RENDERPICTURES];
 
         switch( p_pic->i_status )
         {
@@ -149,11 +140,17 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
                 p_pic->b_top_field_first    = b_top_field_first;
 
                 p_vout->i_heap_size++;
+                p_vout->render.i_last_used_pic =
+                    ( p_vout->render.i_last_used_pic + i_pic + 1 )
+                    % I_RENDERPICTURES;
                 vlc_mutex_unlock( &p_vout->picture_lock );
                 return( p_pic );
 
             case FREE_PICTURE:
                 /* Picture is empty and ready for allocation */
+                p_vout->render.i_last_used_pic =
+                    ( p_vout->render.i_last_used_pic + i_pic + 1 )
+                    % I_RENDERPICTURES;
                 p_freepic = p_pic;
                 break;
 
@@ -167,7 +164,7 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
      */
     if( p_freepic != NULL )
     {
-        vout_AllocatePicture( p_freepic,
+        vout_AllocatePicture( p_vout, p_freepic,
                               p_vout->render.i_width, p_vout->render.i_height,
                               p_vout->render.i_chroma );
 
@@ -194,7 +191,7 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
             p_freepic->i_status = FREE_PICTURE;
             p_freepic = NULL;
 
-            intf_ErrMsg( "vout error: picture allocation failed" );
+            msg_Err( p_vout, "picture allocation failed" );
         }
 
         vlc_mutex_unlock( &p_vout->picture_lock );
@@ -226,8 +223,8 @@ void vout_DestroyPicture( vout_thread_t *p_vout, picture_t *p_pic )
         (p_pic->i_status != RESERVED_DATED_PICTURE) &&
         (p_pic->i_status != RESERVED_DISP_PICTURE) )
     {
-        intf_ErrMsg( "error: picture %p has invalid status %d",
-                     p_pic, p_pic->i_status );
+        msg_Err( p_vout, "picture to destroy %p has invalid status %d",
+                         p_pic, p_pic->i_status );
     }
 #endif
 
@@ -262,7 +259,7 @@ void vout_UnlinkPicture( vout_thread_t *p_vout, picture_t *p_pic )
 
     if( p_pic->i_refcount < 0 )
     {
-        intf_ErrMsg( "vout error: picture refcount is %i", p_pic->i_refcount );
+        msg_Err( p_vout, "picture refcount is %i", p_pic->i_refcount );
         p_pic->i_refcount = 0;
     }
 
@@ -295,7 +292,7 @@ picture_t * vout_RenderPicture( vout_thread_t *p_vout, picture_t *p_pic,
 
     if( p_pic->i_type == DIRECT_PICTURE )
     {
-        if( p_pic->i_refcount )
+        if( !p_vout->render.b_allow_modify_pics || p_pic->i_refcount )
         {
             /* Picture is in a direct buffer and is still in use,
              * we need to copy it to another direct buffer before
@@ -305,7 +302,7 @@ picture_t * vout_RenderPicture( vout_thread_t *p_vout, picture_t *p_pic,
                 /* We have subtitles. First copy the picture to
                  * the spare direct buffer, then render the
                  * subtitles. */
-                vout_CopyPicture( p_pic, PP_OUTPUTPICTURE[0] );
+                CopyPicture( p_vout, p_pic, PP_OUTPUTPICTURE[0] );
 
                 vout_RenderSubPictures( p_vout, PP_OUTPUTPICTURE[0], p_subpic );
 
@@ -333,7 +330,7 @@ picture_t * vout_RenderPicture( vout_thread_t *p_vout, picture_t *p_pic,
         /* Picture is not in a direct buffer, but is exactly the
          * same size as the direct buffers. A memcpy() is enough,
          * then render the subtitles. */
-        vout_CopyPicture( p_pic, PP_OUTPUTPICTURE[0] );
+        CopyPicture( p_vout, p_pic, PP_OUTPUTPICTURE[0] );
 
         vout_RenderSubPictures( p_vout, PP_OUTPUTPICTURE[0], p_subpic );
 
@@ -360,8 +357,10 @@ picture_t * vout_RenderPicture( vout_thread_t *p_vout, picture_t *p_pic,
  * This function will be accessed by plugins. It calculates the relative
  * position of the output window and the image window.
  *****************************************************************************/
-void vout_PlacePicture( vout_thread_t *p_vout, int i_width, int i_height,
-                        int *pi_x, int *pi_y, int *pi_width, int *pi_height )
+void vout_PlacePicture( vout_thread_t *p_vout,
+                        unsigned int i_width, unsigned int i_height,
+                        unsigned int *pi_x, unsigned int *pi_y,
+                        unsigned int *pi_width, unsigned int *pi_height )
 {
     if( (i_width <= 0) || (i_height <=0) )
     {
@@ -413,8 +412,8 @@ void vout_PlacePicture( vout_thread_t *p_vout, int i_width, int i_height,
  * used exactly like a video buffer. The video output thread then manages
  * how it gets displayed.
  *****************************************************************************/
-void vout_AllocatePicture( picture_t *p_pic,
-                           int i_width, int i_height, u32 i_chroma )
+void vout_AllocatePicture( vout_thread_t *p_vout, picture_t *p_pic,
+                           int i_width, int i_height, vlc_fourcc_t i_chroma )
 {
     int i_bytes, i_index;
 
@@ -422,75 +421,95 @@ void vout_AllocatePicture( picture_t *p_pic,
     for( i_index = 0; i_index < VOUT_MAX_PLANES; i_index++ )
     {
         p_pic->p[i_index].p_pixels = NULL;
-        p_pic->p[i_index].b_margin = 0;
-        p_pic->p[i_index].i_pixel_bytes = 1;
+        p_pic->p[i_index].i_pixel_pitch = 1;
     }
 
     /* Calculate coordinates */
     switch( i_chroma )
     {
+        case FOURCC_I410:
+            p_pic->p[ Y_PLANE ].i_lines = i_height;
+            p_pic->p[ Y_PLANE ].i_pitch = i_width;
+            p_pic->p[ Y_PLANE ].i_visible_pitch = p_pic->p[ Y_PLANE ].i_pitch;
+            p_pic->p[ U_PLANE ].i_lines = i_height / 4;
+            p_pic->p[ U_PLANE ].i_pitch = i_width / 4;
+            p_pic->p[ U_PLANE ].i_visible_pitch = p_pic->p[ U_PLANE ].i_pitch;
+            p_pic->p[ V_PLANE ].i_lines = i_height / 4;
+            p_pic->p[ V_PLANE ].i_pitch = i_width / 4;
+            p_pic->p[ V_PLANE ].i_visible_pitch = p_pic->p[ V_PLANE ].i_pitch;
+            p_pic->i_planes = 3;
+            break;
+
         case FOURCC_YV12:
         case FOURCC_I420:
         case FOURCC_IYUV:
             p_pic->p[ Y_PLANE ].i_lines = i_height;
             p_pic->p[ Y_PLANE ].i_pitch = i_width;
-            p_pic->p[ Y_PLANE ].i_visible_bytes = p_pic->p[ Y_PLANE ].i_pitch;
+            p_pic->p[ Y_PLANE ].i_visible_pitch = p_pic->p[ Y_PLANE ].i_pitch;
             p_pic->p[ U_PLANE ].i_lines = i_height / 2;
             p_pic->p[ U_PLANE ].i_pitch = i_width / 2;
-            p_pic->p[ U_PLANE ].i_visible_bytes = p_pic->p[ U_PLANE ].i_pitch;
+            p_pic->p[ U_PLANE ].i_visible_pitch = p_pic->p[ U_PLANE ].i_pitch;
             p_pic->p[ V_PLANE ].i_lines = i_height / 2;
             p_pic->p[ V_PLANE ].i_pitch = i_width / 2;
-            p_pic->p[ V_PLANE ].i_visible_bytes = p_pic->p[ V_PLANE ].i_pitch;
+            p_pic->p[ V_PLANE ].i_visible_pitch = p_pic->p[ V_PLANE ].i_pitch;
             p_pic->i_planes = 3;
             break;
 
         case FOURCC_I422:
             p_pic->p[ Y_PLANE ].i_lines = i_height;
             p_pic->p[ Y_PLANE ].i_pitch = i_width;
-            p_pic->p[ Y_PLANE ].i_visible_bytes = p_pic->p[ Y_PLANE ].i_pitch;
+            p_pic->p[ Y_PLANE ].i_visible_pitch = p_pic->p[ Y_PLANE ].i_pitch;
             p_pic->p[ U_PLANE ].i_lines = i_height;
             p_pic->p[ U_PLANE ].i_pitch = i_width / 2;
-            p_pic->p[ U_PLANE ].i_visible_bytes = p_pic->p[ U_PLANE ].i_pitch;
+            p_pic->p[ U_PLANE ].i_visible_pitch = p_pic->p[ U_PLANE ].i_pitch;
             p_pic->p[ V_PLANE ].i_lines = i_height;
             p_pic->p[ V_PLANE ].i_pitch = i_width / 2;
-            p_pic->p[ V_PLANE ].i_visible_bytes = p_pic->p[ V_PLANE ].i_pitch;
+            p_pic->p[ V_PLANE ].i_visible_pitch = p_pic->p[ V_PLANE ].i_pitch;
             p_pic->i_planes = 3;
             break;
 
         case FOURCC_I444:
             p_pic->p[ Y_PLANE ].i_lines = i_height;
             p_pic->p[ Y_PLANE ].i_pitch = i_width;
-            p_pic->p[ Y_PLANE ].i_visible_bytes = p_pic->p[ Y_PLANE ].i_pitch;
+            p_pic->p[ Y_PLANE ].i_visible_pitch = p_pic->p[ Y_PLANE ].i_pitch;
             p_pic->p[ U_PLANE ].i_lines = i_height;
             p_pic->p[ U_PLANE ].i_pitch = i_width;
-            p_pic->p[ U_PLANE ].i_visible_bytes = p_pic->p[ U_PLANE ].i_pitch;
+            p_pic->p[ U_PLANE ].i_visible_pitch = p_pic->p[ U_PLANE ].i_pitch;
             p_pic->p[ V_PLANE ].i_lines = i_height;
             p_pic->p[ V_PLANE ].i_pitch = i_width;
-            p_pic->p[ V_PLANE ].i_visible_bytes = p_pic->p[ V_PLANE ].i_pitch;
+            p_pic->p[ V_PLANE ].i_visible_pitch = p_pic->p[ V_PLANE ].i_pitch;
             p_pic->i_planes = 3;
             break;
 
         case FOURCC_Y211:
             p_pic->p->i_lines = i_height;
             p_pic->p->i_pitch = i_width;
-            p_pic->p->i_visible_bytes = p_pic->p->i_pitch;
-            p_pic->p->i_pixel_bytes = 4;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 4;
             p_pic->i_planes = 1;
             break;
 
         case FOURCC_YUY2:
             p_pic->p->i_lines = i_height;
             p_pic->p->i_pitch = i_width * 2;
-            p_pic->p->i_visible_bytes = p_pic->p->i_pitch;
-            p_pic->p->i_pixel_bytes = 4;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 4;
+            p_pic->i_planes = 1;
+            break;
+
+        case FOURCC_RGB2:
+            p_pic->p->i_lines = i_height;
+            p_pic->p->i_pitch = i_width;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 1;
             p_pic->i_planes = 1;
             break;
 
         case FOURCC_RV15:
             p_pic->p->i_lines = i_height;
             p_pic->p->i_pitch = i_width * 2;
-            p_pic->p->i_visible_bytes = p_pic->p->i_pitch;
-            p_pic->p->i_pixel_bytes = 2;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 2;
 /* FIXME: p_heap isn't always reachable
             p_pic->p_heap->i_rmask = 0x001f;
             p_pic->p_heap->i_gmask = 0x03e0;
@@ -501,8 +520,8 @@ void vout_AllocatePicture( picture_t *p_pic,
         case FOURCC_RV16:
             p_pic->p->i_lines = i_height;
             p_pic->p->i_pitch = i_width * 2;
-            p_pic->p->i_visible_bytes = p_pic->p->i_pitch;
-            p_pic->p->i_pixel_bytes = 2;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 2;
 /* FIXME: p_heap isn't always reachable
             p_pic->p_heap->i_rmask = 0x001f;
             p_pic->p_heap->i_gmask = 0x07e0;
@@ -513,8 +532,8 @@ void vout_AllocatePicture( picture_t *p_pic,
         case FOURCC_RV32:
             p_pic->p->i_lines = i_height;
             p_pic->p->i_pitch = i_width * 4;
-            p_pic->p->i_visible_bytes = p_pic->p->i_pitch;
-            p_pic->p->i_pixel_bytes = 4;
+            p_pic->p->i_visible_pitch = p_pic->p->i_pitch;
+            p_pic->p->i_pixel_pitch = 4;
 /* FIXME: p_heap isn't always reachable
             p_pic->p_heap->i_rmask = 0xff0000;
             p_pic->p_heap->i_gmask = 0x00ff00;
@@ -523,8 +542,8 @@ void vout_AllocatePicture( picture_t *p_pic,
             break;
 
         default:
-            intf_ErrMsg( "vout error: unknown chroma type 0x%.8x (%4.4s)",
-                         i_chroma, (char*)&i_chroma );
+            msg_Err( p_vout, "unknown chroma type 0x%.8x (%4.4s)",
+                             i_chroma, (char*)&i_chroma );
             p_pic->i_planes = 0;
             return;
     }
@@ -554,16 +573,79 @@ void vout_AllocatePicture( picture_t *p_pic,
     }
 }
 
+/*****************************************************************************
+ * vout_ChromaCmp: compare two chroma values
+ *****************************************************************************
+ * This function returns 1 if the two fourcc values given as argument are
+ * the same format (eg. UYVY / UYNV) or almost the same format (eg. I420/YV12)
+ *****************************************************************************/
+int vout_ChromaCmp( vlc_fourcc_t i_chroma, vlc_fourcc_t i_amorhc )
+{
+    /* If they are the same, they are the same ! */
+    if( i_chroma == i_amorhc )
+    {
+        return 1;
+    }
+
+    /* Check for equivalence classes */
+    switch( i_chroma )
+    {
+        case FOURCC_I420:
+        case FOURCC_IYUV:
+        case FOURCC_YV12:
+            switch( i_amorhc )
+            {
+                case FOURCC_I420:
+                case FOURCC_IYUV:
+                case FOURCC_YV12:
+                    return 1;
+
+                default:
+                    return 0;
+            }
+
+        case FOURCC_UYVY:
+        case FOURCC_UYNV:
+        case FOURCC_Y422:
+            switch( i_amorhc )
+            {
+                case FOURCC_UYVY:
+                case FOURCC_UYNV:
+                case FOURCC_Y422:
+                    return 1;
+
+                default:
+                    return 0;
+            }
+
+        case FOURCC_YUY2:
+        case FOURCC_YUNV:
+            switch( i_amorhc )
+            {
+                case FOURCC_YUY2:
+                case FOURCC_YUNV:
+                    return 1;
+
+                default:
+                    return 0;
+            }
+
+        default:
+            return 0;
+    }
+}
+
 /* Following functions are local */
 
 /*****************************************************************************
- * vout_CopyPicture: copy a picture to another one
+ * CopyPicture: copy a picture to another one
  *****************************************************************************
  * This function takes advantage of the image format, and reduces the
  * number of calls to memcpy() to the minimum. Source and destination
- * images must have same width, height, and chroma.
+ * images must have same width (hence i_visible_pitch), height, and chroma.
  *****************************************************************************/
-static void vout_CopyPicture( picture_t *p_src, picture_t *p_dest )
+static void CopyPicture( vout_thread_t * p_vout,
+                         picture_t *p_src, picture_t *p_dest )
 {
     int i;
 
@@ -571,46 +653,26 @@ static void vout_CopyPicture( picture_t *p_src, picture_t *p_dest )
     {
         if( p_src->p[i].i_pitch == p_dest->p[i].i_pitch )
         {
-            if( p_src->p[i].b_margin )
-            {
-                /* If p_src->b_margin is set, p_dest->b_margin must be set */
-                if( p_dest->p[i].b_hidden )
-                {
-                    /* There are margins, but they are hidden : perfect ! */
-                    FAST_MEMCPY( p_dest->p[i].p_pixels, p_src->p[i].p_pixels,
-                                 p_src->p[i].i_pitch * p_src->p[i].i_lines );
-                    continue;
-                }
-                else
-                {
-                    /* We can't directly copy the margin. Too bad. */
-                }
-            }
-            else
-            {
-                /* Same pitch, no margins : perfect ! */
-                FAST_MEMCPY( p_dest->p[i].p_pixels, p_src->p[i].p_pixels,
-                             p_src->p[i].i_pitch * p_src->p[i].i_lines );
-                continue;
-            }
+            /* There are margins, but with the same width : perfect ! */
+            p_vout->p_vlc->pf_memcpy(
+                         p_dest->p[i].p_pixels, p_src->p[i].p_pixels,
+                         p_src->p[i].i_pitch * p_src->p[i].i_lines );
         }
         else
         {
-            /* Pitch values are different */
-        }
-
-        /* We need to proceed line by line */
-        {
-            u8 *p_in = p_src->p[i].p_pixels, *p_out = p_dest->p[i].p_pixels;
+            /* We need to proceed line by line */
+            uint8_t *p_in = p_src->p[i].p_pixels;
+            uint8_t *p_out = p_dest->p[i].p_pixels;
             int i_line;
 
             for( i_line = p_src->p[i].i_lines; i_line--; )
             {
-                FAST_MEMCPY( p_out, p_in, p_src->p[i].i_visible_bytes );
+                p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                          p_src->p[i].i_visible_pitch );
                 p_in += p_src->p[i].i_pitch;
                 p_out += p_dest->p[i].i_pitch;
             }
         }
     }
+    p_dest->date = p_src->date;
 }
-
